// PROGRAMM FOR PIC-MT-USB - LCD Demo
// PROCESSOR	: PIC16F877A
// CLOCK	: 20MHz, EXTERNAL

#include	<pic.h>   

__CONFIG(UNPROTECT & PWRTEN & HS & BORDIS & WDTDIS & LVPDIS);


#define		E			RD2		
#define		RW			RD1		
#define		RS			RD0		
#define		LIGHT		RD3

#define		CLR_DISP		0x01	
#define		DISP_ON			0x0C	
#define		DISP_OFF		0x08	
#define		CUR_HOME        0x02
#define		CUR_OFF 		0x0C
#define 	CUR_ON_UNDER    0x0E
#define		CUR_ON_BLINK    0x0F
#define 	CUR_LEFT        0x10
#define		CUR_RIGHT       0x14
#define		CUR_UP  		0x80	
#define		CUR_DOWN		0xC0
#define		ENTER           0xC0
#define		DD_RAM_ADDR		0x80	
#define		DD_RAM_ADDR2	0xC0	


#define		DELAY_1000us			498		//498*2+3 = 999us

unsigned  int dly;
unsigned  int i;
unsigned  char data;

unsigned char OlimexMsg[] = " www.olimex.com ";


//delay time = (cntr*2)+3 (in us for 20MHz)
//FULL OPTIMIZATION of C compilator(10 level, enable post-pass optimisation)
void Delay(unsigned int cntr) {									
	for (dly=0; dly!=cntr; dly++) {
		asm("nop");
		asm("nop");	
		asm("nop");	
	}
}

//delay time is ~ 1ms	
void DelayMs (unsigned char e)		
{
	while(--e) {
		Delay(DELAY_1000us);
	 }
}

void E_Pulse(void) {
	E=1;
	asm("nop");
	asm("nop");
	asm("nop");
	E=0;
}

// Init LCD  after reset
void InitLCD(void) {

	// all LCD port is output
	PORTD=0;
	TRISD=0; 								

	RS=0;				
	RW=0;
	DelayMs(110);	
	PORTD=0b00110000;		
	E_Pulse();
	DelayMs(10);	
	PORTD=0b00110000;
	E_Pulse();
	DelayMs(10);
	PORTD=0b00110000;
	E_Pulse();
	DelayMs(10);
	PORTD=0b00100000;
	E_Pulse();
}

// Send char to LCD
LCDSendChar(unsigned char c) {
	
	DelayMs(2);
	// get upper nibble
	data = c & 0b11110000;		
	// send data to LCD
	PORTD = ((PORTD & 0b00001111) | data);				
	// set LCD to write
	RW=0;						
	// set LCD to data mode
	RS=1;						
	E_Pulse();
	// get lower nibble
	data = c & 0b00001111;		
	data = data << 4;
	PORTD = ((PORTD & 0b00001111) | data);
	// set LCD to write
	RW=0;						
	// set LCD to data mode
	RS=1;						
	E_Pulse();
}

// Send command to LCD
LCDSendCmd(unsigned char c) {

	DelayMs(2);
    // get upper nibble
	data = c & 0b11110000;		
	// send data to LCD
	PORTD = ((PORTD & 0b00001111) | data);				
	// set LCD to write
	RW=0;						
	// set LCD to data mode
	RS=0;						
	E_Pulse();
	// get lower nibble
	data = c & 0b00001111;		
	data = data << 4;
	PORTD = ((PORTD & 0b00001111) | data);
	// set LCD to write
	RW=0;						
	// set LCD to data mode
	RS=0;						
	E_Pulse();
}

void LCDSendStr(unsigned char* str) {

	i=0;
	while(str[i]) {
		LCDSendChar(str[i]);
		i++;
	}
}


int main(void) {

	InitLCD();
	LCDSendCmd(DISP_ON);
	LCDSendCmd(CLR_DISP);
	LCDSendStr(OlimexMsg);
	LIGHT = 1;

	while(1) {


	}
}


